
/* slink2ew definitions */

#ifndef __EDATA__
#define __EDATA__



#define MAXMESSAGELEN   160     /* Maximum length of a status or error  */
                                /*   message.                           */
#define MAXRINGNAMELEN  28      /* Maximum length of a ring name.       */
                                /* Should be defined by kom.h           */
#define MAXMODNAMELEN   30      /* Maximum length of a module name      */
                                /* Should be defined by kom.h           */
#define MAXADDRLEN      80      /* Length of SeedLink hostname/address  */

//#include "globalvars.h" 




#define DataBuffer_len	1024


#define Max_nSCNL 		6		// max channels of digitizer (1-4)
#define Max_bytesample	4		// max bytes per sample (1-4)
#define Max_Fsamp		200		// max sampling frequency ()		




#define MaxRxMsgLength	6144	

#define MAX_TTY_LEN 32

// year rollover offset 1024 [weeks] * 7 [days/week] * 86400 [seconds/day]
#define WNRO_offset	619315200	

typedef struct _s_tty
{
	int bitrate;
	char name[MAX_TTY_LEN];
	
} s_tty;

// struttura di configurazione del modulo
typedef struct _s_SysCfg
{
	s_tty	tty;			// parametri di configurazione della porta seriale
	char	StationCode[6];	// nome della stazione
	char	NetworkName[6];
} s_SysCfg;


typedef struct _s_Trace_raw
{
	int		data[Max_Fsamp];	// data buffer (samples)
	char	cmp[4];				// channel name (3 + null)
	char	loc[3];				// location (2 + null)
} s_Trace_raw;





typedef struct _s_Stream_buffer
{
	unsigned long	starttime;				// timestamp del primo campine
	s_Trace_raw 	trace[Max_nSCNL];			// buffer locale, qui ho le tracce nSCNL per le componenti XYZ a cui aggiungo il canale ausiliario (il +1)
	unsigned int 	npts;
} s_Stream_buffer;	// struttura per la traccia RAW contenente i counts, gestiti come intero



// struttura che uso per le tracce di debug
typedef struct
{
	TRACE2_HEADER	header;
	char			databuffer[4096];
} s_TraceBlock;



// nota componenti:
// 1 = verticale
// 2 = verticale + N/S
// 3 = tutte e 3
// le componenti possono essere da 1 a 6, i byte per campione possono essere 3 o 4
// max 192 bytes
typedef struct  __attribute__ ((__packed__))
{
	char MODtxt[4];			// Identifier “MOD<0>“ ASCII 
	long SizeofMOD;			// 184 = size of MOD in bytes (excl. Identifier and size) 
	char DeviceID[12];		// ASCII
	char Version[6];		// ASCII
	char Space_1;			// ASCII space
	char serialNo[4];		// ASCII SN
	char Space_2;			// ASCII space

	unsigned char test_pattern[8];	// binary (hex 22,22,00,55,55,00,ff,ff) 
	unsigned long block_count;	// binary 32 bit block count
	unsigned short Ncomps;	// binary number of components in data
	unsigned short sample_rate;		// samples per block per component
	unsigned short Bytesper;		// bytes per sample to send
	char filter_type[2];			// “LP” linear phase “MP” minimal phase
	unsigned short deci12;			// decimation factor of filter 3
	unsigned short deci10;			// decimation factor of filter 4
	short Plldata;					// 16 bit phase error in 1 second pll
	char Gain_G;					// "G" ASCII 
	unsigned char Gain;					// Binary representation of channel gain.
	long	Cal_gain_C1;			// Component 1 gain value for Hi/Lo gain
	long	Cal_gain_C2;			// Component 2 gain value 
	long	Cal_gain_C3;			// Component 3 gain value 
	long	Offset_C1;				// Component 1 offset value for Hi/lo gain
	long	Offset_C2;			// Component 2 offset value 
	long	Offset_C3;			// Component 3 offset value 	
	short	ADC0;				// Supply Voltage at Supply connector: Supply voltage at input Voltage = (ADC0*10.9/2700)+5 
	short	ADC1;				// Supply current: Supply current. Current (Amps) = ADC1*22/170
	short	ADC2;				// Temp. sensor voltage: Internal temperature Temperature (°Celsius) = ADC2/10-50 
	short	ADC3;				//	Supply Voltage at Remote connector: Supply voltage at remote input. Voltage = (ADC3*10.9/2700)+5 
	short	ADC4;				// User input: User Input:  Input voltage (volts) = ADC4/1000
	short	ADC5;				// User input
	short	ADC6;				// User input
	short	ADC7;				// Not used
	char	coupling[2];		// "DC" ASCII
	unsigned long ltime;		// Time in 32 bit time_t format 
	long	reserved_1;
	long	reserved_2;
	short	reserved_3;			// bit flags of GPS status
	unsigned long GPS_block;	// block count of last valid GPS
	char	GPS_message[72];	// Last valid GPS message
} s_MOD;



// max 192 bytes
typedef struct   __attribute__ ((__packed__))
{
	char MDEtxt[4];			// Identifier “MDE<0>“ ASCII 
	long SizeofMDE;			// 184 = size of MDE in bytes (excl. Identifier and size) 
	char EnhancedSerianNo[8];		// ASCII
	unsigned short deci10_5;			// decimation factor of filter 5	
	unsigned short deci10_6;			// decimation factor of filter 6	
	unsigned short deci10_7;			// decimation factor of filter 7	
	long	Cal_gain_C4;			// Component 4 gain value for Hi/Lo gain	
	long	Cal_gain_C5;			// Component 5 gain value
	long	Cal_gain_C6;			// Component 6 gain value
	long	Offset_C4;				// Component 4 offset value for Hi/lo gain
	long	Offset_C5;				// Component 5 offset value 
	long	Offset_C6;				// Component 6 offset value 
	short	ADC8;					// Input Module 1 Temp sensor 
	short	ADC9;					// Input Module 2 Temp sensor 
	short	ADC10;					// Input Module 3 Temp sensor 
	short	ADC11;					// Input Module 4 Temp sensor 
	short	ADC12;					// Input Module 5 Temp sensor 
	short	ADC13;					// Input Module 6 Temp sensor 
	char	GPS_message[129];		// Last valid GPS message (Trimble)
} s_MDE;





// i canali possono essere da 1 a 6, con formato 3 o 4 bytes
typedef struct  __attribute__ ((__packed__))
{
	char DATtxt[4];			// Identifier “DAT<0>“ ASCII 
	unsigned long SizeofDAT;			// size of DAT in bytes (excl. Identifier and size) 
	unsigned char data[24000];			// non conosco a priori il numero dei canali, non conosco a priori il formato (se a 3 o 4 bytes), massimo 6 canali, 4 bytes, 3000 SPS = 
} s_DATA;



typedef struct  __attribute__ ((__packed__))
{
	char SUMtxt[4];						// Identifier “SUM<0>“ ASCII 
	unsigned long SizeofSUM;			// size of DAT in bytes (excl. Identifier and size) 
	short	reserved;	
	unsigned char	ChecksumLo;
	unsigned char	ChecksumHi;	
	
} s_SUM;




typedef union {
        char          v[4];
        //long          val;
        int32_t         val;
} u_Sample_i4;



typedef union
{
	unsigned int Val;
	struct
	{
		unsigned int Sync:1;		// flag che mi indica che ho sincronizzato la seriale (attivo alto)
		unsigned int Terminate:1;	// flag che mi indica che devo uscire (attivo alto)
		unsigned int MsgValid:1;	// flag che mi indica che devo uscire (attivo alto)
		unsigned int data_MOD:1;	// attivo alto, mi indica che ho un pacchetto MOD
		unsigned int data_MDE:1;	// attivo alto, mi indica che ho un pacchetto MOD
		unsigned int data_DATA:1;	// attivo alto, mi indica che ho un pacchetto MOD
		unsigned int data_SUM:1;	// attivo alto, mi indica che ho un pacchetto MOD
		unsigned int MsgInP:1;		// attivo alto, mi indica che ho una ricezione già in corso
		unsigned int ConfigInit:1;		// attivo alto, mi indica che ho rilevato la configurazione
		unsigned int ChkPass:1;		// attivo alto, mi indica che il checksum è ok
		unsigned int Systime:1;		// attivo alto, per debug
	} flag;
}u_sysStatus;





typedef struct _s_Config
{


	unsigned char Ncomps;	// binary number of components in data
	unsigned short sample_rate;		// samples per block per component
	unsigned char Bytesper;		// bytes per sample to send
	char filter_type[2];			// “LP” linear phase “MP” minimal phase
	unsigned short deci12;			// decimation factor of filter 3
	unsigned short deci10;			// decimation factor of filter 4
	
	
	
	
	
	short Plldata;					// 16 bit phase error in 1 second pll
	
	unsigned char Gain;					// Binary representation of channel gain.
	long	Cal_gain_C1;			// Component 1 gain value for Hi/Lo gain
	long	Cal_gain_C2;			// Component 2 gain value 
	long	Cal_gain_C3;			// Component 3 gain value 
	long	Offset_C1;				// Component 1 offset value for Hi/lo gain
	long	Offset_C2;			// Component 2 offset value 
	long	Offset_C3;			// Component 3 offset value 	
	short	ADC0;				// Supply Voltage at Supply connector: Supply voltage at input Voltage = (ADC0*10.9/2700)+5 
	short	ADC1;				// Supply current: Supply current. Current (Amps) = ADC1*22/170
	short	ADC2;				// Temp. sensor voltage: Internal temperature Temperature (°Celsius) = ADC2/10-50 
	short	ADC3;				//	Supply Voltage at Remote connector: Supply voltage at remote input. Voltage = (ADC3*10.9/2700)+5 

	unsigned long ltime;		// Time in 32 bit time_t format 

	short	reserved_3;			// bit flags of GPS status
	unsigned long GPS_block;	// block count of last valid GPS
	char	GPS_message[72];	// Last valid GPS message
} s_Config;










/*

#define MaxRxMsgLength	24576	// dimensione massima (teorica) dei dati in ingresso







typedef union {
        char          v[4];
        //long          val;
        int32_t         val;
} u_Sample_i4;









typedef union
{
	unsigned int Val;
	struct
	{
		unsigned int Sync:1;		// flag che mi indica che ho sincronizzato la seriale (attivo alto)
		unsigned int Terminate:1;	// flag che mi indica che devo uscire (attivo alto)
		unsigned int MsgValid:1;	// flag che mi indica che devo uscire (attivo alto)
		unsigned int data_MOD:1;	// attivo alto, mi indica che ho un pacchetto MOD
		unsigned int data_MDE:1;	// attivo alto, mi indica che ho un pacchetto MOD
		unsigned int data_DATA:1;	// attivo alto, mi indica che ho un pacchetto MOD
		unsigned int data_SUM:1;	// attivo alto, mi indica che ho un pacchetto MOD
		unsigned int MsgInP:1;		// attivo alto, mi indica che ho una ricezione già in corso
		unsigned int ConfigInit:1;		// attivo alto, mi indica che ho rilevato la configurazione
		unsigned int ChkPass:1;		// attivo alto, mi indica che il checksum è ok
		unsigned int Systime:1;		// attivo alto, per debug
	} flag;
}u_sysStatus;


typedef struct 
{


	unsigned char Ncomps;	// binary number of components in data
	unsigned short sample_rate;		// samples per block per component
	unsigned char Bytesper;		// bytes per sample to send
	char filter_type[2];			// “LP” linear phase “MP” minimal phase
	unsigned short deci12;			// decimation factor of filter 3
	unsigned short deci10;			// decimation factor of filter 4
	
	
	
	
	
	short Plldata;					// 16 bit phase error in 1 second pll
	
	unsigned char Gain;					// Binary representation of channel gain.
	long	Cal_gain_C1;			// Component 1 gain value for Hi/Lo gain
	long	Cal_gain_C2;			// Component 2 gain value 
	long	Cal_gain_C3;			// Component 3 gain value 
	long	Offset_C1;				// Component 1 offset value for Hi/lo gain
	long	Offset_C2;			// Component 2 offset value 
	long	Offset_C3;			// Component 3 offset value 	
	short	ADC0;				// Supply Voltage at Supply connector: Supply voltage at input Voltage = (ADC0*10.9/2700)+5 
	short	ADC1;				// Supply current: Supply current. Current (Amps) = ADC1*22/170
	short	ADC2;				// Temp. sensor voltage: Internal temperature Temperature (°Celsius) = ADC2/10-50 
	short	ADC3;				//	Supply Voltage at Remote connector: Supply voltage at remote input. Voltage = (ADC3*10.9/2700)+5 

	unsigned long ltime;		// Time in 32 bit time_t format 

	short	reserved_3;			// bit flags of GPS status
	unsigned long GPS_block;	// block count of last valid GPS
	char	GPS_message[72];	// Last valid GPS message
} s_Config;

#define Max_nSCNL 3	// numero massimo di tracce che vado a leggere (ne uso solo 3 per l'utilizzo con un accelerometro)







typedef struct 
{
	char     sta[TRACE2_STA_LEN];   // sta code we're tracking 	// 5 + null
	char     cmp[TRACE2_CHAN_LEN];  // cmp code we're tracking 	// 3 + nukk
	char     net[TRACE2_NET_LEN];   // net code we're tracking 	// 2 + null
	char     loc[TRACE2_LOC_LEN];   // loc code we're tracking 	// 2 + null
	
	double 	 starttime;				// timestamp del primo campine
	double   endtime;				// timestamp dell'ultimo campione
	unsigned int npts;
	
	double   nextExpectedStart;		// timestamp teorico del prossimo pacchetto
	int databuffer[DataBuffer_len];	// blocco dati
	
	double *base_ptr;			// puntatore base (mi serve davvero? non posso usare direttamente databuffer[0]) lo lascio in caso di utilizzo di buffer
								// esterno con malloc
	double *limit_ptr;			// puntatore limite (stesso discorso di base_ptr)
	double *read_ptr;			// puntatore di lettura
	double *write_ptr;			// puntatore di scrittura
	
	
	
	
} s_Trace;

*/


#endif   /* __SLINK2EW__  */
