#include <time.h>
#include "earthworm.h"
#include "transport.h"
#include "config.h"
#include "q6602ew.h"
#include "q660_globals.h"
#include "ew_globals.h"

static time_t	Heartbeat_lastBeatTime;		/* time stamp since last heartbeat */
static ew_thread_t Heartbeat_threadID;		/* ID for the hearbeat thread */
static pid_t MyPid;				/* needed for restart message for heartbeat */

/***************************************************************************
 message_send() builds a heartbeat or error message & puts it into
                  shared memory.  Writes errors to log file.
 
*/
void message_send( unsigned char type, short ierr, char *note ) {
    time_t t;
    char message[256];
    long len;

    EW_OtherLogo.type  = type;

    time( &t );
    /* put the message together */
    if( type == TypeHB ) {
       sprintf( message, "%ld %d\n", (long)t, MyPid);
    } else if( type == TypeErr ) {
       sprintf( message, "%ld %hd %s\n", (long)t, ierr, note);
       logit( "et", "%s: %s\n", Q6602EW_NAME, note );
    }
    len = strlen( message );   /* don't include the null byte in the message */

   /* write the message to shared memory */
    if( tport_putmsg( EW_pRegion, &EW_OtherLogo, len, message ) != PUT_OK ) {
        if( type == TypeHB ) {
           logit("et","%s:  Error sending heartbeat.\n", Q6602EW_NAME );
        }
        else if( type == TypeErr ) {
           logit("et","%s:  Error sending error:%d.\n", Q6602EW_NAME, ierr );
        }
    }

   return;
}

thr_ret Heartbeat(void *unused_arg) {
  time_t now;
  MyPid = getpid();	/* set it once on entry */
  message_send( TypeHB, 0, "");
  while(!EW_ShutMeDown) {
    sleep_ew(1000);
    time(&now);
    if (difftime(now, Heartbeat_lastBeatTime) > (double) gConfig.HeartbeatInt) {
      time(&Heartbeat_lastBeatTime);
      message_send( TypeHB, 0, "");
    }
  }
  return THR_NULL_RET;
}

void start_heartbeat(void) {
  // start our heartbeat
  time(&Heartbeat_lastBeatTime);
  StartThread( Heartbeat, 8192, &Heartbeat_threadID );
}
