#if defined(WIN32) || defined(WIN64)
#define _USE_MATH_DEFINES
#endif

#include <math.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <earthworm.h>
#include <chron3.h>
#include <read_arc.h>
#include <transfer.h>

#include "pick_gm.h"

#define MAXDATA     15250

#define NSD 1
#define NSP 3
//static double spectDamp[NSD] = { 0.05 };

void response_spectrum(double *inacc, int npts, double *periods, int nper, double dt, double damping, double *psa, STATION *Sta, GPARM *Gparm, int method);
int spectrum_fft(double *inacc, int npts, double *periods, int nper, double dt, double damping, double *psa, STATION *Sta, GPARM *Gparm);
int spectrum_newmark_beta(double *inacc, int npts, double *periods, int nper, double dt, double damping, double *psa, STATION *Sta, GPARM *Gparm);
int spectrum_nigam_jennings(double *inacc, int npts, double *periods, int nper, double dt, double damping, double *psa, STATION *Sta, GPARM *Gparm);
typedef int(*spec_func_t)  (double *inacc, int npts, double *periods, int nper, double dt, double damping, double *psa, STATION *Sta, GPARM *Gparm);

/* methodology */
spec_func_t spectrum_method[] = {
	  spectrum_fft
	, spectrum_newmark_beta
	, spectrum_nigam_jennings
};

extern int gma(double input[], long npts, double deltat, ResponseStruct *origRS,
	double fTaper[4], double tTaper, long *pPadLen, long *pnfft,
	double spectPer[], int nsp, double spectDamp[], int nsd,
	double output[], long outBufLen, double *work, double *workFFT);

void response_spectrum(double *inacc, int npts, double *periods, int nper, double dt, double damping, double *psa, STATION *Sta, GPARM *Gparm, int method)
{
	spec_func_t spec_func = spectrum_method[method];

	if (!nper) return;

	psa[0] = psa[1] = psa[2] = 0.0;
	spec_func(inacc, npts, periods, nper, dt, damping, psa, Sta, Gparm);
}

int spectrum_fft(double *inacc, int npts, double *periods, int nper, double dt, double damping, double *psa, STATION *Sta, GPARM *Gparm)
{
	// Empty, Not used
	return (0);
}

int spectrum_newmark_beta(double *inacc, int npts, double *periods, int nper, double dt, double damping, double *psa, STATION *Sta, GPARM *Gparm)
{
	int k, i;
	double omega;
	double cval;
	double kval;
	double maxvalue;
	double acc[MAXDATA];
	double vel[MAXDATA];
	double dis[MAXDATA];
	double abacc[MAXDATA];

	for (k = 0; k < nper; k++) {
		omega = 2.0 * M_PI / periods[k];
		maxvalue = 0.0;
		cval = damping * 2. * omega;
		kval = pow(omega, 2.);

		memset(acc, 0, sizeof(acc));
		memset(vel, 0, sizeof(vel));
		memset(dis, 0, sizeof(dis));
		memset(abacc, 0, sizeof(abacc));

		acc[0] = vel[0] = dis[0] = abacc[0] = 0.0;

		/* Initial line */
		acc[0] = (-inacc[0] - cval * vel[0]) - (kval * dis[0]);
		abacc[0] = acc[0] + acc[0];

		for (i = 1; i < npts; i++) {
			dis[i] = dis[i - 1] + (dt * vel[i - 1]) + ((pow(dt, 2.) / 2.) * acc[i - 1]);
			acc[i] = (1. / (1. + dt * 0.5 * cval)) * (-inacc[i] - kval * dis[i] - cval * (vel[i - 1] + (dt * 0.5) * acc[i - 1]));
			vel[i] = vel[i - 1] + dt * (0.5 * acc[i - 1] + 0.5 * acc[i]);
			abacc[i] = inacc[i] + acc[i];

			if (fabs(dis[i]) > maxvalue) {
				maxvalue = fabs(dis[i]);
			}
		}

		psa[k] = pow(omega, 2.0) * maxvalue;
	}

	return (0);
}

int spectrum_nigam_jennings(double *inacc, int npts, double *periods, int nper, double dt, double damping, double *psa, STATION *Sta, GPARM *Gparm)
{
	int k, i;
	double omega;
	double omega2;
	double omega3;
	double omega_d;
	double maxvalue;
	double acc[MAXDATA];
	double vel[MAXDATA];
	double dis[MAXDATA];
	double const_f1, const_f2, const_f3, const_f4, const_f5, const_f6;
	double const_e, const_s, const_c;
	double const_g1, const_g2, const_h1, const_h2;
	double dug, z_1, z_2, z_3, z_4;
	double a_val, b_val;

	for (k = 0; k < nper; k++) {
		omega = 2.0 * M_PI / periods[k];
		omega2 = pow(omega, 2.0);
		omega3 = pow(omega, 3.0);
		omega_d = omega * sqrt(1.0 - pow(damping, 2.0));
		const_f1 = (2.0 * damping) / (omega3 * dt);
		const_f2 = (1.0 / omega2);
		const_f3 = damping * omega;
		const_f4 = (1.0 / omega_d);
		const_f5 = const_f3 * const_f4;
		const_f6 = (2.0 * const_f3);
		const_e = exp(-const_f3 * dt);
		const_s = sin(omega_d * dt);
		const_c = cos(omega_d * dt);
		const_g1 = const_e * const_s;
		const_g2 = const_e * const_c;
		const_h1 = (omega_d * const_g2) - (const_f3 * const_g1);
		const_h2 = (omega_d * const_g1) + (const_f3 * const_g2);
		maxvalue = 0.0;

		memset(acc, 0, sizeof(acc));
		memset(vel, 0, sizeof(vel));
		memset(dis, 0, sizeof(dis));

		for (i = 0; i < npts - 1; i++) {
			dug = inacc[i + 1] - inacc[i];
			z_1 = const_f2 * dug;
			z_2 = const_f2 * inacc[i];
			z_3 = const_f1 * dug;
			z_4 = z_1 / dt;

			if (i == 0) {
				b_val = z_2 - z_3;
				a_val = (const_f5 * b_val) + (const_f4 * z_4);
			}
			else {
				b_val = dis[i - 1] + z_2 - z_3;
				a_val = (const_f4 * vel[i - 1]) + (const_f5 * b_val) + (const_f4 * z_4);
			}

			/* time series */
			dis[i] = (a_val * const_g1) + (b_val * const_g2) + z_3 - z_2 - z_1;
			vel[i] = (a_val * const_h1) - (b_val * const_h2) - z_4;
			acc[i] = (-const_f6 * vel[i]) - (omega2 * dis[i]);

			if (fabs(dis[i]) > maxvalue) {
				maxvalue = fabs(dis[i]);
			}
		}

		psa[k] = pow(omega, 2.0) * maxvalue;
	}

	return (0);
}
