#define __CONFIG_C__
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <earthworm.h>
#include <kom.h>
#include "config.h"

#define STATION_LEN    TRACE2_STA_LEN-1  /* max string-length of station code  */
#define CHAN_LEN       TRACE2_CHAN_LEN-1 /* max string-length of component code*/
#define NETWORK_LEN    TRACE2_NET_LEN-1  /* max string-length of network code  */
#define LOC_LEN        TRACE2_LOC_LEN-1  /* max string-length of location code */

int get_seconds_next(char *str, char **endstr);

// probably won't need this but just in case TYPE_STRONG_MOTION_GMX can be processed in the future
int getLogo()
{
   char *str;
   MSG_LOGO *tlogo = NULL;
   int       nlogo = Gparm.nGetLogo;
   tlogo = (MSG_LOGO *)realloc(Gparm.GetLogo, (nlogo+1)*sizeof(MSG_LOGO));
   if (tlogo == NULL)
   {
      logit("e", "report_gm: GetLogo: error reallocing %zu bytes.\n",
            (nlogo+1)*sizeof(MSG_LOGO));
      return -1;
   }
   Gparm.GetLogo = tlogo;

   if ((str=k_str()) != NULL)       /* read instid */
   {
      if (GetInst(str, &(Gparm.GetLogo[nlogo].instid)) != 0)
      {
         logit("e", "report_gm: Invalid installation name <%s>"
                                                                 " in <GetLogo> cmd!\n", str);
         return -1;
      }
      if ((str=k_str()) != NULL)    /* read module id */
      {
         if (GetModId(str, &(Gparm.GetLogo[nlogo].mod)) != 0)
         {
            logit("e", "report_gm: Invalid module name <%s>"
                  " in <GetLogo> cmd!\n", str);
            return -1;
         }
         if ((str=k_str()) != NULL) /* read message type */
         {
            if (strcmp(str,"TYPE_STRONG_MOTION_GM") !=0 &&
                strcmp(str,"TYPE_STRONG_MOTION_GMX")!=0)
            {
               logit("e","report_gm: Invalid message type <%s> in <GetLogo>"
                     " cmd; must be TYPE_STRONG_MOTION_GM or TYPE_STRONG_MOTION_GMX.\n", str);
               return -1;
            }
            if (GetType(str, &(Gparm.GetLogo[nlogo].type)) != 0) {
               logit("e", "report_gm: Invalid message type <%s>"
                     " in <GetLogo> cmd!\n", str);
               return -1;
            }
            Gparm.nGetLogo++;
         } /* end if msgtype */
      } /* end if modid */
   } /* end if instid */
   return 0;
} /* end GetLogo cmd */

// add the windows specified by the string (e.g. "1m 15m 1h")
void add_windows(char *str)
{
   if (str != NULL && *str != '\0')
   {
      char c;
      int index;
      char *endptr;
      int seconds;
      int len = 0;
      // determine number of windows
      int nwindows = 1;
      while ((c = str[len]) != '\0')
      {
         len++;
         if (isspace(c))
         {
            nwindows++;
         }
      }
      // if no previous windows
      if (Gparm.NumOutputWindows == 0)
      {
         Gparm.OutputWindows = calloc(nwindows, sizeof(int));
         if (!Gparm.OutputWindows)
         {
            logit("e", "report_gm: could not allocate OutputWindows <%d>. Exiting.\n", nwindows);
            exit(-1);
         }
         // start with first window
         index = 0;
      }
      else // add to previous windows
      {
         // start after last window
         index = Gparm.NumOutputWindows;
         nwindows += Gparm.NumOutputWindows;
         Gparm.OutputWindows = realloc(Gparm.OutputWindows, nwindows * sizeof(int));
         if (!Gparm.OutputWindows)
         {
            logit("e", "report_gm: could not allocate OutputWindows <%d>. Exiting.\n", nwindows);
            exit(-1);
         }
      }
      Gparm.NumOutputWindows = nwindows;
      for (; index < Gparm.NumOutputWindows; index++)
      {
         seconds = get_seconds_next(str, &endptr);
         if (seconds < Gparm.OutputInterval)
         {
            logit("e", "report_gm: invalid seconds OutputWindows <%d>. Exiting.\n", seconds);
            exit(-1);
         }
         Gparm.OutputWindows[index] = seconds;
         if (Gparm.Debug)
         {
            printf("report_gm: Gparm.OutputWindows[%d] = %d\n", index, seconds);
         }
         str = endptr;
      }
   }
}

void config_error(char *configfile)
{
   logit("e", "report_gm: Error opening configuration file <%s>\n",
          configfile);
   exit(-1);
}

int get_seconds(char *str)
{
   char *endstr;
   return get_seconds_next(str, &endstr);
}

int get_seconds_next(char *str, char **endstr)
{
   int seconds = 0;
   if (str)
   {
      errno = 0; // clear previous error
      seconds = strtol(str, endstr, 10);
      if (errno != 0)
      {
         logit("e", "report_gm: Invalid time interval <%s>: %s\n", str, strerror(errno));
      }
      else if (str != *endstr)
      {
         char c = tolower(**endstr);
         switch (c)
         {
         case 's':
            // already in seconds
            break;
         case 'm':
            seconds *= SECONDS_PER_MINUTE;
            break;
         case 'h':
            seconds *= SECONDS_PER_HOUR;
            break;
         case 'd':
            seconds *= SECONDS_PER_DAY;
            break;
         default:
            logit("e", "report_gm: Invalid time interval <%s>\n", str);
            break;
         }
         (*endstr)++; // skip units character
      }
   }
   return seconds;
}

void init_params()
{
   memset(&Gparm, 0, sizeof(Gparm));
   Gparm.OutputInterval = SECONDS_PER_MINUTE;
   Gparm.useCurrentTimeForWindow = 1;
}

void init_earthworm()
{
   // get our own pid for restart purposes
   Ewparm.MyPid = getpid();
   if (Ewparm.MyPid == -1) {
      logit("e", "report_gm: Error getting my pid; exiting\n");
      exit(-2);
   }
   if (GetLocalInst(&Ewparm.InstId) != 0)
   {
      logit("e",
         "report_gm: error getting local installation id; exiting\n");
      exit(-3);
   }
   if (GetInst( "INST_WILDCARD", &Ewparm.InstIdWild) != 0)
   {
      logit("e", "report_gm: Error getting InstIdWild.\n");
      exit(-4);
   }
   if (GetModId( "MOD_WILDCARD", &Ewparm.ModIdWild) != 0)
   {
      logit("e", "report_gm: Error getting ModIdWild.\n");
      exit(-5);
   }
   if (GetType("TYPE_ERROR", &Ewparm.TypeError) != 0)
   {
      logit("e", "report_gm: Error getting TypeError; exiting\n");
      exit(-6);
   }
   if (GetType("TYPE_HEARTBEAT", &Ewparm.TypeHeartBeat) != 0)
   {
      logit("e", "report_gm: Error getting TYPE_HEARTBEAT; exiting\n");
      exit(-7);
   }
   if (GetType("TYPE_STRONG_MOTION_GM", &Ewparm.TypeStrongMotionGM) != 0)
   {
      logit("e", "report_gm: Error getting TypeStrongMotionGM; exiting\n");
      exit(-8);
   }
}

// initialize the configuration and earthworm parameters
void config_init()
{
   init_params();
   init_earthworm();
}

// process the configuration file commands
int process(char *configfile, int filenumber)
{
   int nfiles = k_open(configfile);
   if (nfiles != filenumber)
   {
      config_error(configfile);
   }
   else
   {
      char *com;
      char *str;

      while (k_rd())                 // Read next line from active file
      {
         com = k_str();              // Get the first token from line
         if (!com) continue;         // ignore blank line
         if (*com == '#') continue;  // ignore comment
         if (*com == '@')            // another configuration file
         {
            char *nextconfigfile = strdup(com + 1);
            process(nextconfigfile, filenumber + 1);
            free(nextconfigfile);
            continue;
         }
         else if (k_its("Debug"))
         {
            Gparm.Debug = k_int();
            if (Gparm.Debug)
            {
               printf("report_gm: Gparm.Debug=%d\n", Gparm.Debug);
            }
         }
         else if (k_its("MyModId"))
         {
            if ((str = k_str()) != NULL)
            {
               if (GetModId(str, &Gparm.MyModId) == -1)
               {
                  logit("e", "report_gm: Invalid MyModId <%s>.\n", str);
                  exit(-1);
               }
            }
            if (Gparm.Debug)
            {
               printf("report_gm: Gparm.MyModId=%d\n", Gparm.MyModId);
            }
         }
         else if (k_its("InRing"))
         {
            if ((str = k_str()) != NULL)
            {
               if ((Gparm.InKey = GetKey(str)) == -1)
               {
                  logit("e", "report_gm: Invalid InRing name <%s>. Exiting.\n", str);
                  exit(-1);
               }
            }
            if (Gparm.Debug)
            {
               printf("report_gm: Gparm.InKey=%ld\n", Gparm.InKey);
            }
         }
         else if (k_its("HeartbeatInt"))
         {
            Gparm.HeartbeatInt = k_int();
            if (Gparm.Debug)
            {
               printf("report_gm: Gparm.HeartbeatInt=%d\n", Gparm.HeartbeatInt);
            }
         }
         else if (k_its("OutputInterval"))
         {
            int seconds = get_seconds(str = k_str());
            if (seconds >= SECONDS_PER_MINUTE)
            {
               if (seconds != Gparm.OutputInterval)
               {
                  Gparm.OutputInterval = seconds;
               }
            }
            else
            {
               logit("e", "report_gm: Invalid OutputInterval <%s>.\n", str);
               exit(-1);
            }
            if (Gparm.Debug)
            {
               printf("report_gm: Gparm.OutputInterval=%d\n", Gparm.OutputInterval);
            }
         }
         else if (k_its("StaFile"))
         {
            str = k_str();
            if ( str!=NULL  &&  strlen(str)<STAFILE_LEN )
            {
               STAFILE *tmp;
               tmp = (STAFILE *)realloc( Gparm.StaFile, (Gparm.nStaFile+1)*sizeof(STAFILE) );
               if( tmp == NULL )
               {
                  logit( "e", "report_gm: Error reallocing Gparm.StaFile. Exiting.\n" );
                  return -1;
               }
               Gparm.StaFile = tmp;
               strcpy( Gparm.StaFile[Gparm.nStaFile].name, str );
               Gparm.StaFile[Gparm.nStaFile].nsta = 0;
               Gparm.nStaFile++;
            } else {
              logit( "e", "report_gm: Invalid StaFile name <%s>; must be 1-%d chars. "
                     "Exiting.\n", str, STAFILE_LEN );
              return -1;
            }
         }
         else if (k_its("OutputWindows"))
         {
            char *str = k_com();
            // skip to space
            while (*str != ' ')
            {
               str++;
            }
            // skip over space
            while (*str == ' ')
            {
               str++;
            }
            add_windows(str);
         }
         else if (k_its("OutputPathname"))
         {
            char *str = k_str();
            if (str && *str)
            {
               if (strlen(str) >= PATH_MAX)
               {
                   logit("e", "report_gm: <%s> invalid OutputPathname in <%s>\n",
                         str, configfile);
                   exit(-1);
               }
               strcpy(Gparm.OutputPathname, str);
               if (Gparm.Debug)
               {
                  printf("report_gm: Gparm.OutputPathname=%s\n", Gparm.OutputPathname);
               }
            }
         }
         else if (k_its("OutputCommand"))
         {
            char *str = k_str();
            if (str && *str)
            {
               if (strlen(str) >= PATH_MAX)
               {
                   logit("e", "report_gm: <%s> invalid OutputCommand in <%s>\n",
                         str, configfile);
                   exit(-1);
               }
               strcpy(Gparm.OutputCommand, str);
               if (Gparm.Debug)
               {
                  printf("report_gm: Gparm.OutputCommand=%s\n", Gparm.OutputCommand);
               }
            }
         }
         else if (k_its("GetLogo"))
         {
            if (getLogo() != 0)
            {
               exit(-1);
            }
         }
         else if (k_its("useCurrentTimeForWindow"))
         {
            Gparm.useCurrentTimeForWindow = k_int();
            if (Gparm.Debug)
            {
               printf("report_gm: Gparm.useCurrentTimeForWindow=%d\n", Gparm.useCurrentTimeForWindow);
            }
         }
         else
         {
            logit("e", "report_gm: <%s> unknown parameter in <%s>\n",
                  com, configfile);
            continue;
         }
         if (k_err())
         {
            logit("e", "report_gm: Bad <%s> command in <%s>.\n", com,
                  configfile);
            break;
         }
      }
   }
   k_close();
   return 0;
}

// load the configuration files
void config_load(char *configfile)
{
   int status = 0;
   config_init();
   process(configfile, 1);
   if (!Gparm.MyModId)
   {
      logit("e", "report_gm: missing <MyModId> command\n");
      status++;
   }
   if (!Gparm.InKey)
   {
      logit("e", "report_gm: missing <InRing> command\n");
      status++;
   }
   if (!Gparm.HeartbeatInt)
   {
      logit("e", "report_gm: missing <HeartbeatInt> command\n");
      status++;
   }
   if (!Gparm.OutputWindows)
   {
      logit("e", "report_gm: missing <OutputWindows> command\n");
      status++;
   }
   if (!Gparm.nStaFile)
   {
      logit("e", "report_gm: missing <StaFile> command\n");
      status++;
   }
   if (status)
   {
      exit(status);
   }
   // if not specified in config
   if (Gparm.nGetLogo == 0)
   {
      // set default logo
      Gparm.nGetLogo = 1;
      Gparm.GetLogo  = (MSG_LOGO *) calloc( Gparm.nGetLogo, sizeof(MSG_LOGO));
      if (Gparm.GetLogo == NULL) {
         logit("e", "report_gm: Error allocating space for GetLogo. Exiting\n");
         exit(-1);
      }
      Gparm.GetLogo[0].instid = Ewparm.InstIdWild;
      Gparm.GetLogo[0].mod    = Ewparm.ModIdWild;
      Gparm.GetLogo[0].type   = Ewparm.TypeStrongMotionGM;
   }
}
