#include <stdio.h>
#include <string.h>
#include <time.h>
#include <earthworm.h>
#include <transport.h>
#include "config.h"

/******************************************************************************
 * status() builds a heartbeat or error message & puts it into                *
 *                   shared memory.  Writes errors to log file & screen.      *
 ******************************************************************************/
void status(time_t timeNow, unsigned char type, short ierr, char *note) {
    MSG_LOGO logo;
    char msg[256];
    long size;

    /* Build the message
     *******************/
    logo.instid = Ewparm.InstId;
    logo.mod = Gparm.MyModId;
    logo.type = type;

    if (type == Ewparm.TypeHeartBeat) {
        sprintf(msg, "%ld %ld\n", (long) timeNow, (long) Ewparm.MyPid);
    } else if (type == Ewparm.TypeError) {
        sprintf(msg, "%ld %hd %s\n", (long) timeNow, ierr, note);
        logit("et", "report_gm: %s\n", note);
    }

    size = strlen(msg); /* don't include the null byte in the message */

    /* Write the message to shared memory
     ************************************/
    if (tport_putmsg(&Ewparm.InRegion, &logo, size, msg) != PUT_OK) {
        if (type == Ewparm.TypeHeartBeat) {
            logit("et", "report_gm:  Error sending heartbeat.\n");
        } else if (type == Ewparm.TypeError) {
            logit("et", "report_gm:  Error sending error:%d.\n", ierr);
        } 
    } else if (Gparm.Debug) {
        printf("report_gm: sent %s: %s",
               type==Ewparm.TypeHeartBeat?"HeartBeat":"Error", msg);
    }

    return;
}
