#define __WRITE_REPORT_C__
#include <errno.h>
#include <string.h>
#include <earthworm.h>
#include "config.h"
#include "gm_data.h"
#include "write_report.h"

int isTimeToReport(time_t timeNow)
{
   // if no previous report
   if (lastStrongMotionGmReport == 0)
   {
      // start timer
      lastStrongMotionGmReport = timeNow;
      return 0;
   }
   return Gparm.OutputInterval &&
          timeNow - lastStrongMotionGmReport >= Gparm.OutputInterval;
}

static void write_station(FILE *stream, time_t timeNow, STATION *Sta)
{
   int i;
   int time_span;
   PG_TABLE_ENTRY largest_pg_values;

   fprintf(stream, "{");
   fprintf(stream, "\"sta\":\"%s\",", Sta->sta);
   fprintf(stream, "\"net\":\"%s\",", Sta->net);
   fprintf(stream, "\"loc\":\"%s\",", Sta->loc);
   fprintf(stream, "\"lat\":%.10g,", Sta->lat);
   fprintf(stream, "\"lon\":%.10g,", Sta->lon);
   fprintf(stream, "\"time\":%.10g,", Sta->time);
   fprintf(stream, "\"latency\":%d,", Sta->latency);
   fprintf(stream, "\"windows\":[");
   for (i = 0; i < Gparm.NumOutputWindows; i++)
   {
      time_span = Gparm.OutputWindows[i];
      get_largest_pg_values(&largest_pg_values, timeNow, time_span, Sta);
      fprintf(stream, "{");
      fprintf(stream, "\"time-span\":%d,", time_span);
      fprintf(stream, "\"pga\": {");
      fprintf(stream, "\"value\":%.12g,", largest_pg_values.pga.value);
      fprintf(stream, "\"chan\":\"%s\",", largest_pg_values.pga.chan);
      fprintf(stream, "\"time\":%.10g", largest_pg_values.pga.time);
      fprintf(stream, "},");
      fprintf(stream, "\"pgv\": {");
      fprintf(stream, "\"value\":%.12g,", largest_pg_values.pgv.value);
      fprintf(stream, "\"chan\":\"%s\",", largest_pg_values.pgv.chan);
      fprintf(stream, "\"time\":%.10g", largest_pg_values.pgv.time);
      fprintf(stream, "}");
      if (i != Gparm.NumOutputWindows - 1)
      {
         fprintf(stream, "},");
      }
      else
      {
         fprintf(stream, "}");
      }
   }
   fprintf(stream, "]}");
   lastStrongMotionGmReport = timeNow;
}

void write_report(time_t timeNow, int Nsta, STATION *StaArray)
{
   int i;
   STATION *Sta;
   int count = 0;
   FILE *stream = stdout;

   if (*Gparm.OutputPathname && strcmp("stdout", Gparm.OutputPathname) != 0)
   {
      errno = 0; // clear previous error
      stream = fopen(Gparm.OutputPathname, "w");
      if (stream == NULL)
      {
         if (errno != 0)
         {
            logit("e", "report_gm: Could not open file (%s): %s\n",
                  Gparm.OutputPathname, strerror(errno));
         }
         else
         {
            logit("e", "report_gm: Could not open file (%s)\n",
                  Gparm.OutputPathname);
         }
         return;
      }
   }
   fprintf(stream, "{ \"generated\":%ld, \"stations\": [", timeNow);
   for ( i = 0; i < Nsta; i++ )
   {
      Sta = StaArray + i;
      if (Sta->last_saved_index != -1)
      {
         if (count > 0)
	 {
            fprintf(stream, ",\n");
	 }
         else
	 {
            fprintf(stream, "\n");
	 }
	 write_station(stream, timeNow, Sta);
	 count++;
      }
   }
   fprintf(stream, "\n]}\n");
   if (stream != stdout)
   {
      fclose(stream);
   }
   else
   {
      fflush(stream);
   }
}
