#include "exec_program.h"
#include "strfuncs.h"
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>  // needed for waitpid
#include <unistd.h>

/**
 * Execute a program and wait until it terminates.
 * @param program the command to run the program.
 * return 0 if success, non-zero otherwise.
 */
int exec_program(char *const program) {
  char **argv = str2argv(program, ' ');
  int retval = exec_programv(argv);
  freeargv(argv);
  return retval;
}

/**
 * Execute a program and wait until it terminates.
 * @param argv the argument vector which is an array of pointers to strings
 * passed to the new program as its command-line arguments, the first should
 * contain the filename associated with the file being executed.
 * @return 0 if success, non-zero otherwise.
 */
int exec_programv(char *const argv[]) {
  int status;
  pid_t ret;
  char *newenviron[] = {NULL};
  pid_t pid = fork();
  if (pid < 0) { // if creation of a child process was unsuccessful
    fprintf(stderr, "error creating child process\n");
    return 1;
  }
  if (pid != 0) { // parent, wait for child process
    while ((ret = waitpid(pid, &status, 0)) == -1) {
      if (errno != EINTR) {
        fprintf(stderr, "error waiting for child process\n");
        return 2;
      }
    }
    if ((ret == 0) || !(WIFEXITED(status) && !WEXITSTATUS(status))) {
      fprintf(stderr, "unexpected child status: %d\n", ret);
    }
  } else {                             // child process
    execve(argv[0], argv, newenviron); // only returns if error
    fprintf(stderr, "error executing command: %s\n", strerror(errno));
    _Exit(127);
  }
  return 0;
}
