/*
 *
 *   Functions for geometric computations
 *
 */


#include "geom.h"


/* Point Inclusion in Polygon Test by W. Randolph Franklin (WRF) algorithm
 * https://wrfranklin.org/Research/Short_Notes/pnpoly.html
 * 
 * The algorithm is defined here as a static function for local use only
 * 
 * geom_area() function will call pnpoly().
 * 
 * Input parameters:
 * nvert: number of sides of the polygon
 * vertx: float array containing the nvert+1 X coordinates of the polygon
 * verty: float array containing the nvert+1 Y coordinates of the polygon
 * testx: X coordinate of the point to check
 * testy: Y coordinate of the point to check
 * 
 * Return value:
 * 	1 if the point lies inside the polygon
 * 	0 if it lies outside
 * 
 */
static int pnpoly(int nvert, float *vertx, float *verty, float testx, float testy)
{
  int i, j, c = 0;
  
  for (i = 0, j = nvert - 1; i < nvert; j = i++) {
    if ( ((verty[i] > testy) != (verty[j] > testy)) &&
     (testx < (vertx[j] - vertx[i]) * (testy - verty[i]) / (verty[j] - verty[i]) + vertx[i]) )
       c = !c;
  }
  
  return c;
}

/* Simple pnpoly() function parser to mantain compatibility accross
 * Earthworm modules.
 * 
 * Parameters:
 * iout: int pointer where function will store result
 * n: number of sides of the polygon
 * x: float array containing the n+1 X coordinates of the polygon
 * y: float array containing the n+1 Y coordinates of the polygon
 * *** NOTE *** the first point of the polygon is duplicated 
 * 				in location n+1 of the x and y arrays
 * u: float pointer holding the X coordinate of the point to check
 * v: float pointer holding the Y coordinate of the point to check
 * 
 * Output:
 * iout = 1 if the point is inside the polygon
 * iout = 0 if the point is outside
 * 
 * Return value:
 * always 0
 */
int geom_area (int *iout, int n, float *x, float *y, float *u, float *v)
{
    *iout = pnpoly(n, x, y, *u, *v);

    return 0;
}
