// for testing
#include "earthworm.h"

#include "latency_scnl.h"
#include "time_ew.h"
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

// convert LATENCY_SCNL_TEXT to a single string
#define LATENCY_SCNL_TEXT_TO_STRING(L)                                         \
  strreplace((L)->starttime, sizeof(LATENCY_SCNL_TEXT) - 1, '\0', ' ')

// convert LATENCY_SCNL_TEXT to separate strings in struct
#define LATENCY_SCNL_TEXT_TO_STRUCT(L)                                         \
  strreplace((L)->starttime, sizeof(LATENCY_SCNL_TEXT) - 1, ' ', '\0')

/**
 * Convert LATENCY_SCNL_BIN to LATENCY_SCNL_INFO
 * @param latency_scnl_bin the LATENCY_SCNL_BIN
 * @param latency_scnl_info the LATENCY_SCNL_INFO
 */
void latencyscnlbin2info(LATENCY_SCNL_BIN *latency_scnl_bin,
                         LATENCY_SCNL_INFO *latency_scnl_info) {
  memset(latency_scnl_info, 0, sizeof(LATENCY_SCNL_INFO));
  latency_scnl_info->starttime = latency_scnl_bin->starttime;
  latency_scnl_info->endtime = latency_scnl_bin->endtime;
  strncpy(latency_scnl_info->sta, latency_scnl_bin->sta,
          sizeof(latency_scnl_bin->sta));
  strncpy(latency_scnl_info->net, latency_scnl_bin->net,
          sizeof(latency_scnl_bin->net));
  strncpy(latency_scnl_info->chan, latency_scnl_bin->chan,
          sizeof(latency_scnl_bin->chan));
  strncpy(latency_scnl_info->loc, latency_scnl_bin->loc,
          sizeof(latency_scnl_bin->loc));
  latency_scnl_info->latency = latency_scnl_bin->latency;
}

/**
 * Convert LATENCY_SCNL_BIN to LATENCY_SCNL_TEXT
 * @param latency_scnl_bin the LATENCY_SCNL_BIN
 * @param latency_scnl_text the LATENCY_SCNL_TEXT
 */
void latencyscnlbin2text(LATENCY_SCNL_BIN *latency_scnl_bin,
                         LATENCY_SCNL_TEXT *latency_scnl_text) {
  memset(latency_scnl_text, 0, sizeof(LATENCY_SCNL_TEXT));
  sprintf(latency_scnl_text->starttime, "%ld",
          (long)(latency_scnl_bin->starttime * 1000. + 0.5));
  sprintf(latency_scnl_text->endtime, "%ld",
          (long)(latency_scnl_bin->endtime * 1000. + 0.5));
  strncpy(latency_scnl_text->sta, latency_scnl_bin->sta,
          sizeof(latency_scnl_bin->sta));
  strncpy(latency_scnl_text->net, latency_scnl_bin->net,
          sizeof(latency_scnl_bin->net));
  strncpy(latency_scnl_text->chan, latency_scnl_bin->chan,
          sizeof(latency_scnl_bin->chan));
  strncpy(latency_scnl_text->loc, latency_scnl_bin->loc,
          sizeof(latency_scnl_bin->loc));
  sprintf(latency_scnl_text->latency, "%d", latency_scnl_bin->latency);
}

/**
 * Convert LATENCY_SCNL_INFO to LATENCY_SCNL_BIN
 * @param latency_scnl_info the LATENCY_SCNL_INFO
 * @param latency_scnl_bin the LATENCY_SCNL_BIN
 */
void latencyscnlinfo2bin(LATENCY_SCNL_INFO *latency_scnl_info,
                         LATENCY_SCNL_BIN *latency_scnl_bin) {
  memset(latency_scnl_bin, 0, sizeof(LATENCY_SCNL_BIN));
  latency_scnl_bin->starttime = latency_scnl_info->starttime;
  latency_scnl_bin->endtime = latency_scnl_info->endtime;
  strncpy(latency_scnl_bin->sta, latency_scnl_info->sta,
          sizeof(latency_scnl_bin->sta));
  strncpy(latency_scnl_bin->net, latency_scnl_info->net,
          sizeof(latency_scnl_bin->net));
  strncpy(latency_scnl_bin->chan, latency_scnl_info->chan,
          sizeof(latency_scnl_bin->chan));
  strncpy(latency_scnl_bin->loc, latency_scnl_info->loc,
          sizeof(latency_scnl_bin->loc));
  latency_scnl_bin->latency = latency_scnl_info->latency;
}

/**
 * Convert LATENCY_SCNL_INFO to LATENCY_SCNL_BIN
 * @param latency_scnl_info the LATENCY_SCNL_INFO
 * @param latency_scnl_text the LATENCY_SCNL_TEXT
 */
void latencyscnlinfo2text(LATENCY_SCNL_INFO *latency_scnl_info,
                          LATENCY_SCNL_TEXT *latency_scnl_text) {
  memset(latency_scnl_text, 0, sizeof(LATENCY_SCNL_TEXT));
  sprintf(latency_scnl_text->starttime, "%ld",
          (long)(latency_scnl_info->starttime * 1000. + 0.5));
  sprintf(latency_scnl_text->endtime, "%ld",
          (long)(latency_scnl_info->endtime * 1000. + 0.5));
  strncpy(latency_scnl_text->sta, latency_scnl_info->sta,
          sizeof(latency_scnl_info->sta));
  strncpy(latency_scnl_text->net, latency_scnl_info->net,
          sizeof(latency_scnl_info->net));
  strncpy(latency_scnl_text->chan, latency_scnl_info->chan,
          sizeof(latency_scnl_info->chan));
  strncpy(latency_scnl_text->loc, latency_scnl_info->loc,
          sizeof(latency_scnl_info->loc));
  sprintf(latency_scnl_text->latency, "%d", latency_scnl_info->latency);
}

/**
 * Convert LATENCY_SCNL_TEXT to LATENCY_SCNL
 * @param latency_scnl_text the LATENCY_SCNL_TEXT
 * @param latency_scnl_info  the LATENCY_SCNL_INFO
 */
void latencyscnltext2info(LATENCY_SCNL_TEXT *latency_scnl_text,
                          LATENCY_SCNL_INFO *latency_scnl_info) {
  memset(latency_scnl_info, 0, sizeof(LATENCY_SCNL_INFO));
  latency_scnl_info->starttime = atol(latency_scnl_text->starttime) / 1000.;
  latency_scnl_info->endtime = atol(latency_scnl_text->endtime) / 1000.;
  strncpy(latency_scnl_info->sta, latency_scnl_text->sta,
          sizeof(latency_scnl_text->sta));
  strncpy(latency_scnl_info->net, latency_scnl_text->net,
          sizeof(latency_scnl_text->net));
  strncpy(latency_scnl_info->chan, latency_scnl_text->chan,
          sizeof(latency_scnl_text->chan));
  strncpy(latency_scnl_info->loc, latency_scnl_text->loc,
          sizeof(latency_scnl_text->loc));
  latency_scnl_info->latency = atoi(latency_scnl_text->latency);
}

/**
 * Parse the line of latency text.
 * @param latency_scnl_text the LATENCY_SCNL_TEXT
 * @param line the line
 * @return the number of  input items successfully matched and assigned
 * (normally 7) or -1 if EOF
 */
int parselatencytext(LATENCY_SCNL_TEXT *latency_scnl_text, char *line) {
  int retVal = 7;
  int len = strlen(line);
  memset(latency_scnl_text, 0, sizeof(LATENCY_SCNL_TEXT));
  if (len == sizeof(LATENCY_SCNL_TEXT) - 1) {
    strncpy((char *)latency_scnl_text, line, len);
    LATENCY_SCNL_TEXT_TO_STRUCT(latency_scnl_text);
  } else {
    retVal = sscanf(line, "%s %s %s %s %s %s %s", latency_scnl_text->starttime,
                    latency_scnl_text->endtime, latency_scnl_text->sta,
                    latency_scnl_text->chan, latency_scnl_text->net,
                    latency_scnl_text->loc, latency_scnl_text->latency);
    if (retVal == EOF) {
      retVal = -1;
    }
  }
  return retVal;
}

/**
 * Read the latency binary packet.
 * @param latency_scnl_info the LATENCY_SCNL_INFO
 * @param fp the file pointer
 * @return 0 if success, -1 if EOF, positive number if error.
 */
int read_latency_bin(LATENCY_SCNL_INFO *latency_scnl_info, FILE *fp) {
  LATENCY_SCNL_BIN latency_scnl_bin;
  int retVal = fread(&latency_scnl_bin, 1, sizeof(LATENCY_SCNL_BIN), fp);
  if (retVal == sizeof(LATENCY_SCNL_BIN)) {
    latencyscnlbin2info(&latency_scnl_bin, latency_scnl_info);
    return 0;
  }
  if (feof(fp)) // if EOF
  {
    retVal = -1;
  } else {
    logit("e", "read_latency_bin only %d of %ld bytes read\n", retVal,
          (long)sizeof(LATENCY_SCNL_BIN));
    retVal = errno;
    // ensure 0 is not returned
    if (!retVal) {
      retVal = EIO;
    }
  }
  memset(latency_scnl_info, 0, sizeof(LATENCY_SCNL_INFO));
  return retVal;
}

/**
 * Read the latency text packet.
 * @param latency_scnl_info the LATENCY_SCNL_INFO
 * @param fp the file pointer
 * @return 0 if success, -1 if EOF, positive number if error.
 */
int read_latency_text(LATENCY_SCNL_INFO *latency_scnl_info, FILE *fp) {
  int retVal;

  const int bufsz = sizeof(LATENCY_SCNL_TEXT) + 1;
  char buffer[sizeof(LATENCY_SCNL_TEXT) + 1] = "";

  LATENCY_SCNL_TEXT latency_scnl_text;

  memset(buffer, 0, bufsz);
  memset(latency_scnl_info, 0, sizeof(LATENCY_SCNL_INFO));
  if (fgets(buffer, bufsz, fp) != NULL) {
    int i, len = 0;
    for (i = 0; i < bufsz && buffer[i] != '\0'; i++) {
      switch (buffer[i]) {
      case 0:
        break;
      case '\r':
      case '\n':
        buffer[i] = '\0';
        break;
      default:
        len++;
        break;
      }
    }
    retVal = parselatencytext(&latency_scnl_text, buffer);
    if (retVal == 7) {
      latencyscnltext2info(&latency_scnl_text, latency_scnl_info);
      return 0;
    } else if (retVal != -1) {
      logit("e", "read_latency_text could not parse a line (%d %s)\n", retVal,
            buffer);
    }
  } else {
    retVal = errno;
    if (retVal) {
      logit("e", "read_latency_text could not read a line (%d %s)\n", retVal,
            strerror(retVal));
    } else {
      retVal = -1;
    }
  }
  return retVal;
}

/**
 * Convert TBUF2 to LATENCY_SCNL_BIN
 * @param trh the TRACE2_HEADER
 * @param latency the latency
 * @param latency_scnl_bin the LATENCY_SCNL_BIN
 */
void tbuf2latencybin(TRACE2_HEADER *trh, int latency,
                     LATENCY_SCNL_BIN *latency_scnl_bin) {
  memset(latency_scnl_bin, 0, sizeof(LATENCY_SCNL_BIN));
  latency_scnl_bin->starttime = trh->starttime;
  latency_scnl_bin->endtime = trh->endtime;
  strncpy(latency_scnl_bin->sta, trh->sta, sizeof(latency_scnl_bin->sta));
  strncpy(latency_scnl_bin->net, trh->net, sizeof(latency_scnl_bin->net));
  strncpy(latency_scnl_bin->chan, trh->chan, sizeof(latency_scnl_bin->chan));
  strncpy(latency_scnl_bin->loc, trh->loc, sizeof(latency_scnl_bin->loc));
  latency_scnl_bin->latency = latency;
}

/**
 * Convert the TBUF2 to text.
 * @param trh the TRACE2_HEADER
 * @param latency the latency
 * @param str the string which should large enough to hold LATENCY_STR_SIZE
 * characters
 * @param fixed non-zero if fixed size
 */
void tbuf2latencystr(TRACE2_HEADER *trh, int latency, char *str, int fixed) {
  memset(str, 0, sizeof(LATENCY_SCNL_TEXT));
  if (fixed) {
    // latency scnl text output
    LATENCY_SCNL_TEXT *latency_scnl_text = (LATENCY_SCNL_TEXT *)str;
    sprintf(latency_scnl_text->starttime, "%ld",
            (long)(trh->starttime * 1000. + 0.5));
    sprintf(latency_scnl_text->endtime, "%ld",
            (long)(trh->endtime * 1000. + 0.5));
    strcpy(latency_scnl_text->sta, trh->sta);
    strcpy(latency_scnl_text->net, trh->net);
    strcpy(latency_scnl_text->chan, trh->chan);
    strcpy(latency_scnl_text->loc, trh->loc);
    sprintf(latency_scnl_text->latency, "%d", latency);
    LATENCY_SCNL_TEXT_TO_STRING(latency_scnl_text);
  } else {
    sprintf(str, "%ld %ld %s %s %s %s %d", (long)(trh->starttime * 1000. + 0.5),
            (long)(trh->endtime * 1000. + 0.5), trh->sta, trh->chan, trh->net,
            trh->loc, latency);
  }
}

/**
 * Write the latency information as text.
 * @param latency_scnl_info the LATENCY_SCNL_INFO
 * @param fp the file pointer, can be stdout
 * @param outputType the output type
 */
void write_latency(LATENCY_SCNL_INFO *latency_scnl_info, FILE *fp,
                   LATENCY_OUTPUT_TYPE outputType) {
  char startTimeBuf[DATESTR23];
  char endTimeBuf[DATESTR23];
  switch (outputType) {
  case LATENCY_OUTPUT_TEXT_EPOCH:
    fprintf(fp, "%f %f %s.%s.%s.%s %f\n", latency_scnl_info->starttime,
            latency_scnl_info->endtime, latency_scnl_info->sta,
            latency_scnl_info->chan, latency_scnl_info->net,
            latency_scnl_info->loc, latency_scnl_info->latency / 1000.);
    break;
  case LATENCY_OUTPUT_TEXT_UTC:
    datestr23(latency_scnl_info->starttime, startTimeBuf, DATESTR23);
    datestr23(latency_scnl_info->endtime, endTimeBuf, DATESTR23);
    break;
  case LATENCY_OUTPUT_TEXT_LOCAL:
    datestr23_local(latency_scnl_info->starttime, startTimeBuf, DATESTR23);
    datestr23_local(latency_scnl_info->endtime, endTimeBuf, DATESTR23);
    break;
  case LATENCY_OUTPUT_TYPE_LATENCY_SCNL_BIN: {
    LATENCY_SCNL_BIN latency_scnl_bin;
    latencyscnlinfo2bin(latency_scnl_info, &latency_scnl_bin);
    fwrite(&latency_scnl_bin, 1, sizeof(LATENCY_SCNL_BIN), fp);
  } break;
  case LATENCY_OUTPUT_LATENCY_SCNL_TEXT: {
    LATENCY_SCNL_TEXT latency_scnl_text;
    latencyscnlinfo2text(latency_scnl_info, &latency_scnl_text);
    LATENCY_SCNL_TEXT_TO_STRING(&latency_scnl_text);
    fprintf(fp, "%s\n", (char *)&latency_scnl_text);
  } break;
  }
  switch (outputType) {
  case LATENCY_OUTPUT_TEXT_UTC:
  case LATENCY_OUTPUT_TEXT_LOCAL:
    fprintf(fp, "%s %s %s.%s.%s.%s %f\n", startTimeBuf, endTimeBuf,
            latency_scnl_info->sta, latency_scnl_info->chan,
            latency_scnl_info->net, latency_scnl_info->loc,
            latency_scnl_info->latency / 1000.);
    break;
  default:
    break;
  }
}

/**
 * Write the latency information as text.
 * @param latency_scnl_bin the LATENCY_SCNL_BIN
 * @param fp the file pointer, can be stdout
 * @param outputType the output type
 */
void write_latency_bin(LATENCY_SCNL_BIN *latency_scnl_bin, FILE *fp,
                       LATENCY_OUTPUT_TYPE outputType) {
  char startTimeBuf[DATESTR23];
  char endTimeBuf[DATESTR23];
  switch (outputType) {
  case LATENCY_OUTPUT_TEXT_EPOCH:
    fprintf(fp, "%f %f %.5s.%.2s.%.3s.%.2s %f\n", latency_scnl_bin->starttime,
            latency_scnl_bin->endtime, latency_scnl_bin->sta,
            latency_scnl_bin->chan, latency_scnl_bin->net,
            latency_scnl_bin->loc, latency_scnl_bin->latency / 1000.);
    break;
  case LATENCY_OUTPUT_TEXT_UTC:
    datestr23(latency_scnl_bin->starttime, startTimeBuf, DATESTR23);
    datestr23(latency_scnl_bin->endtime, endTimeBuf, DATESTR23);
    break;
  case LATENCY_OUTPUT_TEXT_LOCAL:
    datestr23_local(latency_scnl_bin->starttime, startTimeBuf, DATESTR23);
    datestr23_local(latency_scnl_bin->endtime, endTimeBuf, DATESTR23);
    break;
  case LATENCY_OUTPUT_TYPE_LATENCY_SCNL_BIN:
    fwrite(latency_scnl_bin, 1, sizeof(LATENCY_SCNL_BIN), fp);
    break;
  case LATENCY_OUTPUT_LATENCY_SCNL_TEXT: {
    LATENCY_SCNL_TEXT latency_scnl_text;
    latencyscnlbin2text(latency_scnl_bin, &latency_scnl_text);
    LATENCY_SCNL_TEXT_TO_STRING(&latency_scnl_text);
    fprintf(fp, "%s\n", (char *)&latency_scnl_text);
  } break;
  default:
    break;
  }
  switch (outputType) {
  case LATENCY_OUTPUT_TEXT_UTC:
  case LATENCY_OUTPUT_TEXT_LOCAL:
    fprintf(fp, "%s %s %.5s.%.2s.%.3s.%.2s %f\n", startTimeBuf, endTimeBuf,
            latency_scnl_bin->sta, latency_scnl_bin->chan,
            latency_scnl_bin->net, latency_scnl_bin->loc,
            latency_scnl_bin->latency / 1000.);
    break;
  default:
    break;
  }
}
