#include <errno.h>
#include <stdio.h>
#include <sys/stat.h>

#include "libmseed.h"
#include "mseedprocess.h"
#include "tbufprocess.h"

static tbufprocess_input ti;
static char mseed_version      = -1;
static int mseed_buffer_length = 0;
static char mseed_buffer[MS_PACK_DEFAULT_RECLEN];

// message callback handler
// msg - the message data
// msglength - the message length
// version - the message version (MSG_TBUF, MSG_MSEED2, MSG_MSEED3)
// returns 0 if successful
int
message_handler (char *msg, int msglength, char version)
{
  printf ("mseedtest message_handler: msglength=%d, version=%d\n", msglength, version);
  switch (version)
  {
  case MSG_TBUF:
    tbufprocess (&ti, msg);
    break;
  default:
    if (msglength <= 0 || msglength > MS_PACK_DEFAULT_RECLEN)
    {
      printf ("invalid new message length %d\n", msglength);
      msglength = 0;
    }
    else if (!mseed_buffer_length)
    {
      printf ("no previous message\n");
    }
    else
    {
      if (mseed_version != version)
      {
        printf ("new version %d does not match previous version %d\n", version, mseed_version);
        mseed_buffer_length = 0;
      }
      else if (mseed_buffer_length != msglength)
      {
        printf ("new message legnth  %d does not match previous message legnth  %d\n", msglength, mseed_buffer_length);
        mseed_buffer_length = 0;
      }
      else if (memcmp (mseed_buffer, msg, msglength))
      {
        printf ("new message does not match previous message\n");
        mseed_buffer_length = 0;
      }
      else
      {
        printf ("new message matches previous message\n");
      }
    }
    if (msglength && !mseed_buffer_length)
    {
      mseed_buffer_length = msglength;
      mseed_version       = version;
      memcpy (mseed_buffer, msg, msglength);
      printf ("saved message %d\n", msglength);
      if (version == 3)
      {
        if (ti.flags & MSF_PACKVER2)
        {
          ti.flags &= ~MSF_PACKVER2;
        }
      }
      else
      {
        if (!(ti.flags & MSF_PACKVER2))
        {
          ti.flags |= MSF_PACKVER2;
        }
      }
    }
    break;
  }
  return 0;
}

int
main (int argc, char **argv)
{
  int retval;
  struct stat sb = {0};
  FILE *fh;

  if (argc != 2)
  {
    ms_log (2, "%s requires a single file name argument\n", argv[0]);
    return -1;
  }
  /* Read specified file into buffer */
  else if (!(fh = fopen (argv[1], "rb")))
  {
    ms_log (2, "Error opening %s: %s\n", argv[1], strerror (errno));
    return -1;
  }
  else if (fstat (fileno (fh), &sb))
  {
    ms_log (2, "Error stating %s: %s\n", argv[1], strerror (errno));
    fclose (fh);
    return -1;
  }
  else
  {
    size_t bufferlength = sb.st_size;
    char *buffer        = (char *)malloc (bufferlength);
    if (!buffer)
    {
      ms_log (2, "Error allocating buffer of %" PRIsize_t " bytes\n",
              (bufferlength >= 0) ? bufferlength : 0);
      return -1;
    }
    retval = fread (buffer, 1, bufferlength, fh);
    fclose (fh);
    if (retval != bufferlength)
    {
      ms_log (2, "Error reading file\n");
      return -1;
    }
    tbufprocess_init (&ti);
    ti.mh      = message_handler;
    ti.verbose = 1;
    retval     = mseedprocess (buffer, bufferlength, 0, message_handler, message_handler);
    if (retval)
    {
      printf ("mseedprocess error: %d\n", retval);
    }

    tbufprocess (&ti, NULL); // flush

    free (buffer);
    tbufprocess_msr3_free (&ti);
  }

  return 0;
}
